//+------------------------------------------------------------------+
//|                                              Hull moving average |
//|                                                          HMA.mq4 |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1 Yellow
#property indicator_color2 DeepSkyBlue
#property indicator_color3 DeepSkyBlue
#property indicator_color4 PaleVioletRed
#property indicator_color5 PaleVioletRed
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2
#property indicator_width5 2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    HMAPeriod   = 15;
extern int    HMAPrice    = PRICE_CLOSE;
extern int    HMAMethod   = MODE_LWMA;
extern bool   Interpolate = true;
//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];
double buffer4[];
double buffer5[];
double buffer6[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   calculatingHMA = false;
bool   returningBars  = false;


//+------------------------------------------------------------------
//|                                                                 |
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   HMAPeriod  = MathMax(2,HMAPeriod);
   IndicatorBuffers(6);
      SetIndexBuffer(0,buffer1);
      SetIndexBuffer(1,buffer2);
      SetIndexBuffer(2,buffer3);
      SetIndexBuffer(3,buffer4);
      SetIndexBuffer(4,buffer5);
      SetIndexBuffer(5,buffer6);
         timeFrame = stringToTimeFrame(TimeFrame);
         IndicatorFileName = WindowExpertName();

      if (TimeFrame=="calculateHMA") { calculatingHMA = true; return(0); }            
      if (TimeFrame=="returnBars")   { returningBars = true;  return(0); }            

      //
      //
      //
      //
      //
   
   IndicatorShortName("HMA "+timeFrameToString(timeFrame)+" ("+HMAPeriod+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int limit,i,counted_bars = IndicatorCounted();
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;   
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returningBars)  { buffer1[0] = limit+1;  return(0); }
           
   //
   //
   //
   //
   //
              
   if (calculatingHMA || timeFrame==Period())
   {
      if (!calculatingHMA && buffer1[limit] > buffer1[limit+1]) CleanPoint(limit,buffer2,buffer3);
      if (!calculatingHMA && buffer1[limit] < buffer1[limit+1]) CleanPoint(limit,buffer4,buffer5);
    
         int HalfPeriod = MathFloor(HMAPeriod/2);
         int HullPeriod = MathFloor(MathSqrt(HMAPeriod));
        
         //
         //
         //
         //
         //
          
         for(i=limit; i>=0; i--) buffer6[i] = iMA(NULL,0,HalfPeriod,0,HMAMethod,HMAPrice,i)*2.0-iMA(NULL,0,HMAPeriod ,0,HMAMethod,HMAPrice,i);
         for(i=limit; i>=0; i--)
         {
            buffer1[i] = iMAOnArray(buffer6,0,HullPeriod,0,HMAMethod,i);
            if (!calculatingHMA)
            {
               buffer2[i] = EMPTY_VALUE;
               buffer3[i] = EMPTY_VALUE;
               buffer4[i] = EMPTY_VALUE;
               buffer5[i] = EMPTY_VALUE;
               if (buffer1[i] > buffer1[i+1]) PlotPoint(i,buffer2,buffer3,buffer1);
               if (buffer1[i] < buffer1[i+1]) PlotPoint(i,buffer4,buffer5,buffer1);
            }               
         }               
      return(0);
   }

   //
   //
   //
   //
   //
            
      limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));

      if (buffer1[limit] > buffer1[limit+1]) CleanPoint(limit,buffer2,buffer3);
      if (buffer1[limit] < buffer1[limit+1]) CleanPoint(limit,buffer4,buffer5);
   	for(i = limit; i >= 0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
   
            buffer1[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateHMA",HMAPeriod,HMAPrice,HMAMethod,0,y);
            buffer2[i] = EMPTY_VALUE;
            buffer3[i] = EMPTY_VALUE;
            buffer4[i] = EMPTY_VALUE;
            buffer5[i] = EMPTY_VALUE;

            if(!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

            //
            //
	         //
            //
            //
		 
            datetime time = iTime(NULL,timeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++)
                  buffer1[i+k] = buffer1[i] +(buffer1[i+n]-buffer1[i])*k/n;
      }
   	for(i = limit; i >= 0; i--)
      {	
         if (buffer1[i] > buffer1[i+1]) PlotPoint(i,buffer2,buffer3,buffer1);
         if (buffer1[i] < buffer1[i+1]) PlotPoint(i,buffer4,buffer5,buffer1);
      }                  
   
   //
   //
   //
   //
   //
      
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
      if (first[i+2] == EMPTY_VALUE) {
          first[i]    = from[i];
          first[i+1]  = from[i+1];
          second[i]   = EMPTY_VALUE;
         }
      else {
          second[i]   = from[i];
          second[i+1] = from[i+1];
          first[i]    = EMPTY_VALUE;
         }
      }
   else
      {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}